@@ -21,11 +21,6 @@
 
 #include "readtex.h"
 
-#ifdef XMESA
-#include "GL/xmesa.h"
-static int fullscreen = 1;
-#endif
-
 static int WIDTH = 640;
 static int HEIGHT = 480;
 
@@ -273,13 +268,6 @@ key(unsigned char k, int x, int y)
       v -= 0.01;
       break;
 
-#ifdef XMESA
-   case ' ':
-      fullscreen = (!fullscreen);
-      XMesaSetFXmode(fullscreen ? XMESA_FX_FULLSCREEN : XMESA_FX_WINDOW);
-      break;
-#endif
-
    case '+':
       LODbias--;
       break;
