@@ -103,7 +103,8 @@ static void init(void)
    theNurb = gluNewNurbsRenderer();
    gluNurbsProperty(theNurb, GLU_SAMPLING_TOLERANCE, 25.0);
    gluNurbsProperty(theNurb, GLU_DISPLAY_MODE, GLU_FILL);
-   gluNurbsCallback(theNurb, GLU_ERROR, nurbsError);
+   gluNurbsCallback(theNurb, GLU_ERROR,
+                    (void (GLAPIENTRY *)(void))nurbsError);
 }
 
 static void display(void)
