@@ -9,7 +9,7 @@
 #include <assert.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include "GL/glew.h"
+#include "glad/glad.h"
 #include "glut_wrap.h"
 
 static GLuint Window = 0;
@@ -208,7 +208,7 @@ int main( int argc, char *argv[] )
    glutInitDisplayMode( GLUT_RGB | GLUT_DEPTH | GLUT_DOUBLE );
 
    Window = glutCreateWindow("Texture Objects");
-   glewInit();
+   gladLoadGL();
    if (!Window) {
       exit(1);
    }
