@@ -5,7 +5,7 @@
 #include <math.h>
 #include <string.h>
 
-#include <GL/glew.h>
+#include <glad/glad.h>
 #include "glut_wrap.h"
 
 
@@ -155,7 +155,7 @@ main(int argc, char *argv[])
 	exit(1);
     }
 
-    glewInit();
+    gladLoadGL();
     Init();
 
     glutReshapeFunc(Reshape);
