@@ -34,13 +34,10 @@
  * Brian Paul  26 January 2000
  */
 
-#define GLX_GLXEXT_PROTOTYPES
-#define GL_GLEXT_PROTOTYPES
-
 #include <assert.h>
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
-#include <GL/gl.h>
+#include "glad/glad.h"
 #include <GL/glx.h>
 #include <stdio.h>
 #include <string.h>
@@ -103,7 +100,17 @@ struct visual_attribs
    int swapMethod;
 };
 
-   
+struct options
+{
+   InfoMode mode;
+   GLboolean findBest;
+   GLboolean limits;
+   GLboolean singleLine;
+   char *displayName;
+   GLboolean allowDirect;
+};
+
+
 /**
  * Version of the context that was created
  *
@@ -459,6 +466,13 @@ print_screen_info(Display *dpy, int scrnum,
 		       visinfo->visual, mask, &attr);
 
    if (glXMakeCurrent(dpy, win, ctx)) {
+      int loaded_gl = gladLoadGLLoader((GLADloadproc) glXGetProcAddressARB);
+
+      if (!loaded_gl) {
+         fprintf(stderr, "Error: unable to load GL functions\n");
+         exit(1);
+      }
+
       const char *serverVendor = glXQueryServerString(dpy, scrnum, GLX_VENDOR);
       const char *serverVersion = glXQueryServerString(dpy, scrnum, GLX_VERSION);
       const char *serverExtensions = glXQueryServerString(dpy, scrnum, GLX_EXTENSIONS);
@@ -479,11 +493,11 @@ print_screen_info(Display *dpy, int scrnum,
       CheckError(__LINE__);
 
       /* Get some ext functions */
-      extfuncs.GetProgramivARB = (GETPROGRAMIVARBPROC)
+      extfuncs.GetProgramivARB = (PFNGLGETPROGRAMIVARBPROC)
          glXGetProcAddressARB((GLubyte *) "glGetProgramivARB");
-      extfuncs.GetStringi = (GETSTRINGIPROC)
+      extfuncs.GetStringi = (PFNGLGETSTRINGIPROC)
          glXGetProcAddressARB((GLubyte *) "glGetStringi");
-      extfuncs.GetConvolutionParameteriv = (GETCONVOLUTIONPARAMETERIVPROC)
+      extfuncs.GetConvolutionParameteriv = (PFNGLGETCONVOLUTIONPARAMETERIVPROC)
          glXGetProcAddressARB((GLubyte *) "glGetConvolutionParameteriv");
 
       if (!glXQueryVersion(dpy, & glxVersionMajor, & glxVersionMinor)) {
@@ -560,28 +574,26 @@ print_screen_info(Display *dpy, int scrnum,
 
       CheckError(__LINE__);
 
-#ifdef GL_VERSION_2_0
       if (version >= 20) {
          char *v = (char *) glGetString(GL_SHADING_LANGUAGE_VERSION);
          printf("%s shading language version string: %s\n", oglstring, v);
       }
-#endif
+
       CheckError(__LINE__);
-#ifdef GL_VERSION_3_0
+
       if (version >= 30 && !es2Profile) {
          GLint flags;
          glGetIntegerv(GL_CONTEXT_FLAGS, &flags);
          printf("%s context flags: %s\n", oglstring, context_flags_string(flags));
       }
-#endif
+
       CheckError(__LINE__);
-#ifdef GL_VERSION_3_2
+
       if (version >= 32 && !es2Profile) {
          GLint mask;
          glGetIntegerv(GL_CONTEXT_PROFILE_MASK, &mask);
          printf("%s profile mask: %s\n", oglstring, profile_mask_string(mask));
       }
-#endif
 
       CheckError(__LINE__);
 
@@ -1278,6 +1290,73 @@ find_best_visual(Display *dpy, int scrnum)
 }
 
 
+static void
+usage(void)
+{
+   printf("Usage: glxinfo [-v] [-t] [-h] [-b] [-l] [-s] [-i] [-display <dname>]\n");
+   printf("\t-display <dname>: Print GLX visuals on specified server.\n");
+   printf("\t-i: Force an indirect rendering context.\n");
+   printf("\t-B: brief output, print only the basics.\n");
+   printf("\t-v: Print visuals info in verbose form.\n");
+   printf("\t-t: Print verbose visual information table.\n");
+   printf("\t-h: This information.\n");
+   printf("\t-b: Find the 'best' visual and print its number.\n");
+   printf("\t-l: Print interesting OpenGL limits.\n");
+   printf("\t-s: Print a single extension per line.\n");
+}
+
+
+static void
+parse_args(int argc, char *argv[], struct options *options)
+{
+   int i;
+
+   options->mode = Normal;
+   options->findBest = GL_FALSE;
+   options->limits = GL_FALSE;
+   options->singleLine = GL_FALSE;
+   options->displayName = NULL;
+   options->allowDirect = GL_TRUE;
+
+   for (i = 1; i < argc; i++) {
+      if (strcmp(argv[i], "-display") == 0 && i + 1 < argc) {
+         options->displayName = argv[i + 1];
+         i++;
+      }
+      else if (strcmp(argv[i], "-i") == 0) {
+         options->allowDirect = GL_FALSE;
+      }
+      else if (strcmp(argv[i], "-t") == 0) {
+         options->mode = Wide;
+      }
+      else if (strcmp(argv[i], "-v") == 0) {
+         options->mode = Verbose;
+      }
+      else if (strcmp(argv[i], "-B") == 0) {
+         options->mode = Brief;
+      }
+      else if (strcmp(argv[i], "-b") == 0) {
+         options->findBest = GL_TRUE;
+      }
+      else if (strcmp(argv[i], "-l") == 0) {
+         options->limits = GL_TRUE;
+      }
+      else if (strcmp(argv[i], "-h") == 0) {
+         usage();
+         exit(0);
+      }
+      else if(strcmp(argv[i], "-s") == 0) {
+         options->singleLine = GL_TRUE;
+      }
+      else {
+         printf("Unknown option `%s'\n", argv[i]);
+         usage();
+         exit(0);
+      }
+   }
+}
+
+
 int
 main(int argc, char *argv[])
 {
